from setuptools import setup, find_packages

setup(
    name = 'kwak',
    packages=find_packages(),
    version='1.1.0',
    url='https://github.com/alexxromero/kwak_wavelets',
    author='Ben G. Lillard and Alexis Romero',
    author_email='alexir2@uci.edu',
    license='MIT',
    description='Statistical tool for wavelet analysis',
    )
